import { Publish } from "electron-builder-http/out/publishOptions";
import { Arch, ArchType } from "electron-builder-util";
export declare type TargetConfigType = Array<string | TargetConfig> | string | TargetConfig | null;
export interface TargetConfig {
    /**
     * The target name. e.g. `snap`.
     */
    readonly target: string;
    /**
     * The arch or list of archs.
     */
    readonly arch?: Array<ArchType> | ArchType;
}
export declare class Platform {
    name: string;
    buildConfigurationKey: string;
    nodeName: string;
    static MAC: Platform;
    static LINUX: Platform;
    static WINDOWS: Platform;
    static OSX: Platform;
    constructor(name: string, buildConfigurationKey: string, nodeName: string);
    toString(): string;
    createTarget(type?: string | Array<string> | null, ...archs: Array<Arch>): Map<Platform, Map<Arch, Array<string>>>;
    static current(): Platform;
    static fromString(name: string): Platform;
}
export declare abstract class Target {
    readonly name: string;
    readonly isAsyncSupported: boolean;
    readonly abstract outDir: string;
    readonly abstract options: TargetSpecificOptions | null | undefined;
    constructor(name: string, isAsyncSupported?: boolean);
    abstract build(appOutDir: string, arch: Arch): Promise<any>;
    finishBuild(): Promise<any>;
}
export interface TargetSpecificOptions {
    /**
     The [artifact file name pattern](https://github.com/electron-userland/electron-builder/wiki/Options#artifact-file-name-pattern).
     */
    readonly artifactName?: string | null;
    readonly publish?: Publish;
}
export declare const DEFAULT_TARGET = "default";
export declare const DIR_TARGET = "dir";
export declare type CompressionLevel = "store" | "normal" | "maximum";
export interface BeforeBuildContext {
    readonly appDir: string;
    readonly electronVersion: string;
    readonly platform: Platform;
    readonly arch: string;
}
export interface SourceRepositoryInfo {
    type?: string;
    domain?: string;
    user: string;
    project: string;
}
